@@ -6,6 +6,8 @@ use std::convert::TryFrom;
 use std::fmt;
 use std::io::{self, Read as _, Write as _};
 use std::net::Shutdown;
+#[cfg(not(async_net_no_io_safety))]
+use std::os::unix::io::{AsFd, BorrowedFd, OwnedFd};
 #[cfg(unix)]
 use std::os::unix::io::{AsRawFd, RawFd};
 #[cfg(windows)]
@@ -172,6 +174,22 @@ impl AsRawFd for UnixListener {
     }
 }
 
+#[cfg(all(not(async_net_no_io_safety), unix))]
+impl AsFd for UnixListener {
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.inner.get_ref().as_fd()
+    }
+}
+
+#[cfg(all(not(async_net_no_io_safety), unix))]
+impl TryFrom<OwnedFd> for UnixListener {
+    type Error = io::Error;
+
+    fn try_from(value: OwnedFd) -> Result<Self, Self::Error> {
+        Self::try_from(std::os::unix::net::UnixListener::from(value))
+    }
+}
+
 #[cfg(windows)]
 impl AsRawSocket for UnixListener {
     fn as_raw_socket(&self) -> RawSocket {
@@ -371,6 +389,22 @@ impl AsRawFd for UnixStream {
     }
 }
 
+#[cfg(all(not(async_net_no_io_safety), unix))]
+impl AsFd for UnixStream {
+    fn as_fd(&self) -> BorrowedFd<'_> {
+        self.inner.get_ref().as_fd()
+    }
+}
+
+#[cfg(all(not(async_net_no_io_safety), unix))]
+impl TryFrom<OwnedFd> for UnixStream {
+    type Error = io::Error;
+
+    fn try_from(value: OwnedFd) -> Result<Self, Self::Error> {
+        Self::try_from(std::os::unix::net::UnixStream::from(value))
+    }
+}
+
 #[cfg(windows)]
 impl AsRawSocket for UnixStream {
     fn as_raw_socket(&self) -> RawSocket {
